#lang racket

(require vigracket)
(require (rename-in 2htdp/image
                    (save-image save-plt-image)
                    (image-width plt-image-width)
                    (image-height plt-image-height)))
(require lang/posn)

;; HELPERS
;; -----------------------------------------------------------------------------
;; Bilder prozentual skalieren
(define (resizeimage-percent img p)
  (resizeimage img 
               (inexact->exact (round (* (image-width img)  (/ p 100))))
               (inexact->exact (round (* (image-height img) (/ p 100)))) 
               2))

;;Thresholding
(define (threshold v t [mark0 0.0] [mark1 255.0])
  (if (< v t)
      mark0
      mark1))

;;Find the region with the most pixels assigned to
(define (largestRegionID features [idx 0] [max_idx 0])
  (if (= idx (image-height features))
      max_idx
      (largestRegionID features
                       (+ idx 1)
                       (if (> (image-ref features 0 idx 0) (image-ref features 0 max_idx 0))
                           idx
                           max_idx))))

;;Uses the above function to find the bounding box of the largest region
(define (largestRegionBBox image)
  (let* ((labels (labelimage image))
         (features (extractfeatures image labels))
         (max_region_id (largestRegionID features 2 2)))
    (vector (inexact->exact (image-ref features 1 max_region_id 0)) ;left
            (inexact->exact (image-ref features 2 max_region_id 0)) ;upper
            (inexact->exact (image-ref features 3 max_region_id 0)) ;right
            (inexact->exact (image-ref features 4 max_region_id 0)) ;lower
            )))

;;Overlaying of BBoxes onto image
(define (overlay-bboxes img bboxes colors)
  (if (empty? bboxes)
       (image->bitmap img)
       (let ((bbox  (car bboxes))
             (color (car colors)))
         (underlay/xy (overlay-bboxes img (cdr bboxes) (cdr colors))
                      (- (vector-ref bbox 0) 1)
                      (- (vector-ref bbox 1) 1)
                      (rectangle (+ (- (vector-ref bbox 2) (vector-ref bbox 0)) 2)
                                 (+ (- (vector-ref bbox 3) (vector-ref bbox 1)) 2)
                                 'outline color)))))


;; Main program
;; -----------------------------------------------------------------------------
(define image_dir (current-directory))

(define img (load-image (build-path image_dir "IMG_8553.JPG")))

(define resized_img (resizeimage-percent img 10))

(define work_img (image->green resized_img))
(show-image work_img "Working Image")

;(show-image (cannyedgeimage work_img 1.0 0.0 255.0))
;(show-image (cannyedgeimage work_img 2.0 0.0 255.0))
;(show-image (cannyedgeimage work_img 3.0 0.0 255.0))

;(show-image (cannyedgeimage work_img 1.0 1.0 255.0))
;(show-image (cannyedgeimage work_img 1.0 5.0 255.0))
;(show-image (cannyedgeimage work_img 1.0 20.0 255.0))

(define canny_img (cannyedgeimage work_img 1.0 20.0 255.0))
(show-image canny_img "Canny Edge Detection")

;(show-image (image-map (curryr threshold 100.0) work_img))
;(show-image (image-map (curryr threshold 150.0) work_img))
;(show-image (image-map (curryr threshold 200.0) work_img))

(define threshold_img (image-map (curryr threshold 150.0) work_img))
(show-image threshold_img "Thresholding")

(define bbox1 (largestRegionBBox threshold_img)) 
(define bbox2 (largestRegionBBox canny_img))

(define result1 (bitmap->rgbimage (overlay-bboxes resized_img (list bbox1) '(green))))
(define result2 (bitmap->rgbimage (overlay-bboxes resized_img (list bbox2) '(red))))

(define crop1_img (subimage resized_img (vector-ref bbox1 0)(vector-ref bbox1 1)(vector-ref bbox1 2)(vector-ref bbox1 3)))
(define crop2_img (subimage resized_img (vector-ref bbox2 0)(vector-ref bbox2 1)(vector-ref bbox2 2)(vector-ref bbox2 3)))

(show-image result1 "Result using Thresholding")
(show-image result2 "Result using Canny")