#lang racket
(require vigracket)
(require (rename-in 2htdp/image
                    (save-image save-plt-image)
                    (image-width plt-image-width)
                    (image-height plt-image-height)))



;; HELPERS
;; -----------------------------------------------------------------------------

;;Image showing/saving
(define present-image (lambda (img name) (show-image img name)))
;(define present-image void)
;(define present-image (lambda (img name) (save-image img (build-path (current-directory) name))))

;;Thresholding
(define (threshold v t [mark0 0.0] [mark1 255.0])
  (if (< v t)
      mark0
      mark1))

;;Find the region with the most pixels assigned to
(define (largestRegionID features [idx 0] [max_idx 0])
  (if (= idx (image-height features))
      max_idx
      (largestRegionID features
                       (+ idx 1)
                       (if (> (image-ref features 0 idx 0) (image-ref features 0 max_idx 0))
                           idx
                           max_idx))))

;;Uses the above function to find the bounding box of the largest region
(define (largestRegionBBox image)
  (let* ((labels (labelimage image))
         (features (extractfeatures image labels))
         (max_region_id (largestRegionID features 2 2)))
    (vector (inexact->exact (image-ref features 1 max_region_id 0)) ;left
            (inexact->exact (image-ref features 2 max_region_id 0)) ;upper
            (inexact->exact (image-ref features 3 max_region_id 0)) ;right
            (inexact->exact (image-ref features 4 max_region_id 0)) ;lower
            )))

;;Overlaying of BBoxes onto image
(define (overlay-bboxes img bboxes colors)
  (if (empty? bboxes)
       (image->bitmap img)
       (let ((bbox  (car bboxes))
             (color (car colors)))
         (underlay/xy (overlay-bboxes img (cdr bboxes) (cdr colors))
                      (- (vector-ref bbox 0) 1)
                      (- (vector-ref bbox 1) 1)
                      (rectangle (+ (- (vector-ref bbox 2) (vector-ref bbox 0)) 2)
                                 (+ (- (vector-ref bbox 3) (vector-ref bbox 1)) 2)
                                 'outline color)))))

;; Main program
;; -----------------------------------------------------------------------------
(define image_dir (current-directory))
(define img (load-image (build-path image_dir "reversi-image.png")))
(define img_gray (image->green img))
(present-image img_gray "image_gray.png")

(present-image (image-map (curryr threshold 50) img_gray)  "reversi-thresh-t050.png")
(present-image (image-map (curryr threshold 100) img_gray) "reversi-thresh-t100.png")

(define thresh_img (image-map (curryr threshold 50) img_gray))

(define thresh_bbox (largestRegionBBox thresh_img)) 

(present-image (cannyedgeimage img_gray 3.0  1.0 255.0) "reversi-canny-s3-t1.png")
(present-image (cannyedgeimage img_gray 3.0  2.0 255.0) "reversi-canny-s3-t2.png")

(define canny_img (cannyedgeimage img 3.0  20.0 255.0))

(define canny_bbox (largestRegionBBox canny_img)) 


(present-image (bitmap->rgbimage (overlay-bboxes img (list canny_bbox) '(green))) "reversi-canny-bbox.png")
(present-image (bitmap->rgbimage (overlay-bboxes img (list thresh_bbox) '(red)))  "reversi-thresh-bbox.png")

(define canny_crop (subimage img  (vector-ref canny_bbox 0) (vector-ref canny_bbox 1) (vector-ref canny_bbox 2) (vector-ref canny_bbox 3)))
(define thresh_crop (subimage img  (vector-ref thresh_bbox 0) (vector-ref thresh_bbox 1) (vector-ref thresh_bbox 2) (vector-ref thresh_bbox 3)))
                  
(present-image canny_crop  "reversi-canny-crop.png")
(present-image thresh_crop "reversi-thresh-crop.png")

(define (findFirstPixelInRow img x1 x2 row)
  (let ((not_found 
         (= (apply max (image-ref img x1 row)) 
            0)))
    (if not_found
        (if (= x1 x2)
            #f
            (findFirstPixelInRow
              img 
              (+ x1 (sgn (- x2 x1))) x2 row))
        x1)))

(define canny_bbox-width (-  (vector-ref canny_bbox 2)  (vector-ref canny_bbox 0)))
(define canny_corner-size (round (* canny_bbox-width 0.025)))

(define canny_left  (- (findFirstPixelInRow canny_img (vector-ref canny_bbox 0) (vector-ref canny_bbox 2) (vector-ref canny_bbox 1)) (vector-ref canny_bbox 0)))
(define canny_right (- (findFirstPixelInRow canny_img (vector-ref canny_bbox 2) (vector-ref canny_bbox 0) (vector-ref canny_bbox 1)) (vector-ref canny_bbox 0)))

(define canny_pos (+ (/ (+ canny_left canny_right) 2) canny_corner-size))
(define canny_angle  (/ (* (atan  (- canny_bbox-width canny_pos) canny_pos ) -180) pi))

                          
                          
(define thresh_bbox-width (-  (vector-ref thresh_bbox 2)  (vector-ref thresh_bbox 0)))
(define thresh_corner-size (round (* thresh_bbox-width 0.025)))

(define thresh_left  (- (findFirstPixelInRow thresh_img (vector-ref thresh_bbox 0) (vector-ref thresh_bbox 2) (vector-ref thresh_bbox 1)) (vector-ref thresh_bbox 0)))
(define thresh_right  (- (findFirstPixelInRow thresh_img (vector-ref thresh_bbox 2) (vector-ref thresh_bbox 0) (vector-ref thresh_bbox 1)) (vector-ref thresh_bbox 0)))
  
(define thresh_pos (+ (/ (+ thresh_left thresh_right) 2) thresh_corner-size))
(define thresh_angle  (/ (* (atan thresh_pos (- thresh_bbox-width thresh_pos)) 180) pi))


(define canny_crop-rotated (rotateimage canny_crop canny_angle 1))
(define thresh_crop-rotated (rotateimage thresh_crop thresh_angle 1))

(present-image canny_crop-rotated  "reversi-canny-crop-rotated.png")
(present-image thresh_crop-rotated "reversi-thresh-crop-rotated.png")

(define canny_bbox2  (largestRegionBBox (cannyedgeimage canny_crop-rotated 3.0  20.0 255.0)))
(define thresh_bbox2 (largestRegionBBox (image-map (curryr threshold 100) (list (car thresh_crop-rotated)))))


(define canny_crop2 (subimage canny_crop-rotated  (vector-ref canny_bbox2 0) (vector-ref canny_bbox2 1) (vector-ref canny_bbox2 2) (vector-ref canny_bbox2 3)))
(define thresh_crop2 (subimage thresh_crop-rotated  (vector-ref thresh_bbox2 0) (vector-ref thresh_bbox2 1) (vector-ref thresh_bbox2 2) (vector-ref thresh_bbox2 3)))

(present-image canny_crop2 "reversi-canny-crop2.png")
(present-image thresh_crop2 "reversi-thresh-crop2.png")


;;From here on: proceed with canny
(define empty_board_state (make-list 8 (make-list 8 'empty)))

(define step_x (/ (image-width canny_crop2) 8))
(define step_y (/ (image-height canny_crop2) 8))

(define (board_pos->color image x y)
  (image-ref image             
            (inexact->exact (round (+ (/ step_x 2) (* x step_x))))
            (inexact->exact (round (+ (/ step_y 2) (* y step_y))))))

(define (classify-color col)
  (let* ((intensity (/ (apply + col) (length col))))
      (if (> (first col) (* 2 intensity))
            'red
            (if (> (second col) (* 1.5 intensity))
                  'green
                  'empty))))

(define (board_rows image x y)
  (if (> y 7) '()
      (cons (board_cols image x y) 
            (board_rows image x (+ y 1)))))

(define (board_cols image x y)
  (if (> x 7) '()
      (cons (classify-color (board_pos->color image x y)) (board_cols image (+ x 1) y))))

(define board_state (board_rows canny_crop2 0 0))
board_state